import { describe, it, expect, vi, beforeEach, afterEach } from "vitest";
import { $log } from "@tsed/common";

vi.mock("@tsed/common", () => ({
  $log: {
    appenders: {
      set: vi.fn()
    }
  }
}));

describe("Logger Configuration", () => {
  const originalEnv = process.env;

  beforeEach(() => {
    vi.resetModules();
    vi.clearAllMocks();
    process.env = { ...originalEnv };
  });

  afterEach(() => {
    process.env = originalEnv;
    vi.restoreAllMocks();
  });

  it("should not set appenders in non-production environment", async () => {
    process.env.NODE_ENV = "development";
    await import("./index");
    expect($log.appenders.set).not.toHaveBeenCalled();
  });

  it("should set appenders in production environment", async () => {
    process.env.NODE_ENV = "production";
    await import("./index");

    expect($log.appenders.set).toHaveBeenCalledTimes(2);
    expect($log.appenders.set).toHaveBeenCalledWith("stdout", {
      type: "stdout",
      levels: ["info", "debug"],
      layout: {
        type: "json"
      }
    });
    expect($log.appenders.set).toHaveBeenCalledWith("stderr", {
      levels: ["trace", "fatal", "error", "warn"],
      type: "stderr",
      layout: {
        type: "json"
      }
    });
  });

  it("should export correct DILoggerOptions in non-production", async () => {
    process.env.NODE_ENV = "development";
    const { default: loggerOptions } = await import("./index");
    expect(loggerOptions).toEqual({
      disableRoutesSummary: false
    });
  });

  it("should export correct DILoggerOptions in production", async () => {
    process.env.NODE_ENV = "production";
    const { default: loggerOptions } = await import("./index");
    expect(loggerOptions).toEqual({
      disableRoutesSummary: true
    });
  });
});
