import { Inject, Injectable } from "@tsed/di";
import { User } from "../entities/User";
import { SqliteDatasource } from "../datasources/SqliteDatasource";
import { DataSource, Repository } from "typeorm";

type ProfileData = {
  service: string;
  serviceIdentifier: string;
  username?: string;
  emails?: string[];
  accessToken?: string;
};

@Injectable({
  deps: [SqliteDatasource]
})
export class UserService {
  private userRepository: Repository<User>;

  constructor(@Inject(SqliteDatasource) private dataSource: DataSource) {
    this.userRepository = this.dataSource.getRepository(User);
  }

  public async findOrCreate(profileData: ProfileData): Promise<User> {
    let user = await this.userRepository.findOne({
      where: { serviceIdentifier: profileData.serviceIdentifier }
    });

    if (!user) {
      user = this.userRepository.create({
        service: profileData.service,
        serviceIdentifier: profileData.serviceIdentifier,
        username: profileData.username,
        emails: profileData.emails,
        accessToken: profileData.accessToken
      });
    } else {
      user.emails = profileData.emails || user.emails;
      user.accessToken = profileData.accessToken || user.accessToken;
    }

    return this.userRepository.save(user);
  }

  public async getUserById(id: string): Promise<User | null> {
    return this.userRepository.findOne({
      where: { id },
      relations: ["links"]
    });
  }

  public async getAllUsers(): Promise<User[]> {
    return this.userRepository.find();
  }
}
